(** Une interface fonctionnelle  l'outil Zenity
    Dernire version ici: http://besson.qc.to/publis/Zenity/
    @author Lilian Besson <lilian.besson[AT]ens-cachan.fr>
*)

exception ZenityOcaml_Erreur_Annule of int
(** L'exception qui est renvoy si le processus appel par les
    fonctions suivantes retourne un code d'erreur UNIX spcifiant une
    erreur. *)

val ecrire_dans_fichier : chaine:string -> adresse:string -> unit
(** Ecrit la chaine [chaine] dans le fichier contenu  l'adresse
    relative ou absolue [adresse]. Lve une exception si ce fichier
    est absent. *)

val nom_fichier_temporaire : string
(** Nom du fichier temporaire ou est stock le rsultat de l'appel
     Zenity. *)

val fairezenity : string -> unit
(** Appel zenity avec une commande, et fork le rsultat dans
    nom_fichier_temporaire*)

(** {6 Dfinition des diffrentes fonctions utiles du module ZenityOcaml } *)
(** Toutes ces fonctions ont des paramtres uniquement optionnels,
    qui sont initialiss avec les valeurs par dfaut.  Chaque
    fonction a au moins les paramtres communs [~title],
    [~window_icon], [~width], [~height], [~expiration]. *)


(** {7 zenity --calendar} *)

val calendar :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int ->
  ?text:string ->
  ?day:int ->
  ?month:int -> ?year:int -> ?date_format:string -> unit -> string
(** @param title Fixe le titre de la fentre GTK+ cre par la
    fonction.  @param window_icon Adresse relative ou absolue o
    aller chercher une image d'icone pour la fentre (en bta).
    @param width Largeur de la fentre. En dehors d'une certaine
    plage de valeurs, n'a plus d'effets.  @param expiration Fixe
    un temps en secondes avant la fermeture automatique de la
    fentre. Pas encore support pour toutes les fonctions (en
    bta).  @param text Fixe le message qui s'affiche dans la
    boite de dialogue.  @param day Numro du jour
    pr-slectionn dans la boite de dialogue (entre 1 et 31).
    @param month Idem pour le mois (entre 1 et 12).  @param year
    Idem pour l'anne.  @param date_format Not for casual
    users. *)
(** Ouvre une demande de slection de date. *)

(** {7 zenity --entry} *)

val entry :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int ->
  ?text:string -> ?entry_text:string -> ?hide_text:bool -> unit -> string
(** @param entry_text Fixe le texte dj crit par dfaut dans la
    boite de dialogue.  @param hide_text Si prsent et vrai (=
    [true]) le texte est cach lors de la frappe. *)
(** Ouvre une demande de texte. *)


(** {7 zenity --password} *)

val password :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int -> ?expiration:int -> ?username:string -> unit -> string
(** @param username Nom de l'utilisateur auquel on demande le mot de
    passe (facultatif). *)
(** Ouvre une demande de mot de passe. *)


type add_forms =
    Add_entry of string
  (** Ajoute une entre requte de texte. *)
  | Add_password of string
  (** Ajoute une entre requte de mot de passe. *)
  | Add_calendar of string
  (** Ajoute une entre requte de date. *)
  | NoAdd
(** N'ajoute rien. *)
(** Type des entres supplmentaires de la fonction forms
    (formulaire). *)

val print_add_forms : add_forms -> string
(** Une mthode simple pour afficher des lments de ce type. *)

(** {7 zenity --forms} *)

val forms :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int ->
  ?text:string ->
  ?separator:string ->
  ?forms_date_format:string -> ?add:add_forms list -> unit -> string
(** @param add Liste des lments ajouts au formulaire. L'ordre
    compte, et la liste peut etre nulle.  @param separator Inutile
    [pas visible]. *)
(** Ouvre un formulaire. Voir [add_forms] pour plus de dtails. *)


(** {7 zenity --color-selection} *)

val color_selection :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int -> ?color:Graphics.color -> unit -> Graphics.color
(** @param color Fixe la couleur choisie par dfaut pour la fentre
    de demande de slection de couleur. *)
(** Ouvre une demande de slection de couleur. Ncessite le module
    Graphics, inclus dans les versions compiles zenity.cma et
    zenity.cmxa. Il n'y a normalement rien d'autre  faire pour
    l'utilisation de ZenityOcaml. *)

(** {7 zenity --scale} *)

val scale :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?text:string ->
  ?value:int ->
  ?min_value:int ->
  ?max_value:int ->
  ?step:int -> ?print_partial:bool -> ?hide_value:bool -> unit -> int
(** @param value Fixe la valeur choisie par dfaut pour la fentre
    de demande de choix de valeur.  @param min_value Fixe la
    valeur minimale qu'il sera possible de choisir.  @param
    max_value Fixe la valeur maximale qu'il sera possible de
    choisir.  @param step Fixe le pas de l'interval dans lequel il
    sera possible de choisir.  @param hide_value Si prsent et
    vrai (= [true]) la valeur courante est cache lors de la
    frappe.  @param print_partial Undocumented. *)
(** Ouvre une demande de valeurs entires. *)


(** {7 zenity --file-selection} *)

val file_selection :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int ->
  ?filename:string ->
  ?multiple:bool ->
  ?directory:bool ->
  ?save:bool ->
  ?separator:string ->
  ?confirm_overwrite:bool -> ?file_filter:string -> unit -> string list
(** @param filename Nom du fichier surlequel la slection est faite
    par dfaut.  @param multiple Si prsent et vrai (= [true])
    la slection autorise plusieurs fichiers.  @param directory
    Si prsent et vrai (= [true]) la slection n'autorise que la
    slection des dossiers.  @param file_filter Expression
    rgulire bash faisant office de filtre sur les fichiers
    slectionnables. *)
(** Ouvre une demande de slection de fichier dans un explorateur de dossiers. *)


(** {7 zenity --list} *)

val list :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int ->
  ?text:string ->
  ?column:string ->
  ?checklist:bool ->
  ?radiolist:bool ->
  ?separator:string ->
  ?multiple:bool ->
  ?editable:bool ->
  ?print_column:int ->
  ?hide_column:int ->
  ?hide_header:bool -> ?liste:string list -> unit -> string list
(** @param column Item inscrit dans la colonne.  @param liste Liste,
    vide ou non, des lments dans la colonne.  @param multiple Si
    prsent et vrai (= [true]) la slection autorise plusieurs
    choix.  @param checklist Si prsent et vrai (= [true]) la
    slection autorise plusieurs choix en cochant. *)
(** Ouvre une demande de slection dans une liste construite par
    l'utilisateur (argument liste). Le paramtre separator est
    inutile. Les paramtres non documents ne sont pas encore bien
    supports (bta). *) (* TODO : fix it *)


(** {7 zenity --text-info} *)

val text_info :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int ->
  ?filename:string ->
  ?editable:bool ->
  ?display:int ->
  ?html:bool ->
  ?url:string -> ?checkbox:string -> ?font:string -> unit -> string
(** @param display Prcise le serveur X a utiliser pour afficher la
    fentre GTK.  @param editable Permet de rendre le contenu de
    la fentre ditable.  @param checkbox Fixe le message de la
    boite de confirmation.  @param filename Renseigne l'adresse
    relative ou absolue du fichier a afficher.  @param font
    Renseigne la police  utiliser pour afficher le contenu de la
    boite. Une liste complte des polices supportes devrait
    voir le jour bientt.  @param html Doit etre actif pour
    utiliser url, qui affiche le contenu d'une page web (ou d'un
    document en renseignant son adresse URI).  @param url
    Renseigne l'adresse de la page web a afficher.  *)
(** Ouvre une fentre informative contenant du texte. *)


(** {7 Edit} *)
  
val edit :
  string ->
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int -> ?display:int -> ?font:string -> unit -> string
(** Fournit un diteur de texte [minimal].  Le premier paramtre
    sans nom correspond  l'adresse relative ou absolue du fichier 
    diter. Bien sur, les fonctionnalits d'un tel diteur sont
    trs limites. *)

(** {7 zenity --error} *)

val error :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int -> ?text:string -> ?no_wrap:bool -> unit -> unit
(** Ouvre une fentre d'erreur. Lve une exception si l'utilisateur
    n'appuie pas sur le bouton 'positif'. *)


(** {7 zenity --info} *)

val info :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int -> ?text:string -> ?no_wrap:bool -> unit -> unit
(** Ouvre une fentre informative. Lve une exception si
    l'utilisateur n'appuie pas sur le bouton 'positif'. *)


(** {7 zenity --question} *)

val question :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int ->
  ?text:string ->
  ?ok_label:string -> ?cancel_label:string -> ?no_wrap:bool -> unit -> unit
(** @param ok_label Fixe le contenu du message de la boite 'positive'
    de confirmation.  @param cancel_label Fixe le contenu du
    message de la boite 'ngative' d'annulation.  *)
(** Ouvre une demande de rponse  une question. Lve une exception
    si l'utilisateur n'appuie pas sur le bouton 'positif'. *)


(** {7 zenity --warning} *)

val warning :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int -> ?text:string -> ?no_wrap:bool -> unit -> unit
(** Ouvre une fentre d'avertissement. Lve une exception si
    l'utilisateur n'appuie pas sur le bouton 'positif'.  *)


(** {7 zenity --notification} *)

val notification :
  ?title:string ->
  ?window_icon:string ->
  ?width:int ->
  ?height:int ->
  ?expiration:int -> ?text:string -> ?listen:bool -> unit -> int
(** @param listen Not for casual users. *)
(** Ouvre une fentre d'avertissement. Ne fait rien si text est
    vide. *)


type urgency_level = Low | Normal | Critical
(** Rprsente les trois niveaux d'urgence utilises pour la
    fonction notify. *)
val print_urgency_level : urgency_level -> string

(** {7 notify} *)

val notify :
  ?urgency:urgency_level ->
  ?expiration:int ->
  ?icon:string -> ?body:string -> ?summary:string -> unit -> unit
(** @param urgency Niveau d'urgence de la notification.  @param body
    Contenu du message de la notification.  @param summary Titre
    de la notification. *)
(** Affiche une notification systme, dans la zone prvue  cet
    effet. summary doit etre non vide. *)
  
  
val help : string -> string
(** Affiche l'aide de zenity concernant le topic argument. *)

